==> luna3-2.13.89-py3-none-any.whl: luna3/lunavl/mixins/matcher.py <==
"""
Module realize matcher mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub
from luna3.python_matcher.match_objects import BinaryReference, Candidates, Reference


class MatcherMixin(MixinStub):
    """Matcher Mixin to work with matcher."""

    def matchFaces(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchFaces

        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, attributes, faces)
            accountIdFilter: account id to match faces
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching (failed and succeeded matches)

        >>> import json
        >>> from testData.resources import FACE_WARP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.python_matcher.match_objects import FaceFilters
        >>> from luna3.python_matcher.match_objects import Reference
        >>> from luna3.python_matcher.match_objects import Candidates
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(sampleIds=[sampleId], raiseError=True).json[0]["attribute_id"]
        >>> faceId = client.createFace(attributeId=attributeId, raiseError=True).json["face_id"]
        >>> response = client.matchFaces(
        ...     candidates=[Candidates(FaceFilters(faceIds=[faceId]), limit=3, threshold=0.5)],
        ...     references=[Reference("face", faceId)]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        [
            {
                "matches": [
                    {
                        "filters": {
                            "face_ids": [...],
                            "origin": "faces"
                        },
                        "result": [
                            {
                                "face": {
                                    "account_id": ...,
                                    "avatar": "",
                                    "create_time": ...,
                                    "event_id": null,
                                    "external_id": "",
                                    "face_id": ...,
                                    "lists": [],
                                    "user_data": ""
                                },
                                "similarity": 1.0
                            }
                        ]
                    }
                ],
                "reference": {
                    "id": ...,
                    "type": "face"
                }
            }
        ]

        """
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/faces", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

    match = matchFaces  # COMPAT

    def matchBodies(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for human bodies similar to a
        given reference(s) by matching them. A body attribute descriptor should be extracted for
        reference(s) and candidate(s).

        The sources for references are events or binary descriptor. The sources for candidates is events.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchBodies

        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: event, sdk_descriptor, raw_descriptor)
            accountIdFilter: account id to match bodies
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching (failed and succeeded matches).

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP
        >>> from luna3.common.http_objs import Policies, DetectPolicy, ExtractPolicy
        >>> from luna3.python_matcher.match_objects import EventFilters
        >>> from luna3.python_matcher.match_objects import Reference
        >>> from luna3.python_matcher.match_objects import Candidates
        >>>
        >>> detectPolicy = DetectPolicy(detectBody=1, detectFace=0)
        >>> extractPolicy = ExtractPolicy(extractBodyDescriptor=1, extractFaceDescriptor=0)
        >>> policies = Policies(detectPolicy=detectPolicy, extractPolicy=extractPolicy)
        >>> handlerId = client.createHandler(
        ...     description="happy_people", policies=policies, raiseError=True
        ... ).json["handler_id"]
        >>> eventId = client.emitEvents(
        ...     handlerId=handlerId, inputData=BODY_WARP, raiseError=True,
        ... ).json['events'][0]['event_id']
        >>> response = client.matchBodies(
        ...     candidates=[Candidates(EventFilters(eventIds=[eventId]), limit=1, threshold=0.5)],
        ...     references=[Reference("event", eventId)]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        [
            {
                "matches": [
                    {
                        "filters": {
                            "event_ids": [...],
                            "origin": "events"
                        },
                        "result": [
                            {
                                "event": {
                                    "accessories": null,
                                    "account_id": ...,
                                    "age": null,
                                    "attach_result": [],
                                    "body_basic_attributes": null,
                                    "body_detections": [
                                        {
                                            "detect_time": ...,
                                            "detect_ts": null,
                                            "detection": {
                                                "rect": {
                                                    "height": ...,
                                                    "width": ...,
                                                    "x": ...,
                                                    "y": ...
                                                }
                                            },
                                            "image_origin": null,
                                            "sample_id": ...
                                        }
                                    ],
                                    "create_time": ...,
                                    "deepfake": null,
                                    "emotion": null,
                                    "end_time": ...,
                                    "ethnic_group": null,
                                    "event_id": ...,
                                    "external_id": "",
                                    "face_detections": null,
                                    "face_id": null,
                                    "gender": null,
                                    "handler_id": ...,
                                    "liveness": null,
                                    "location": {
                                        "area": null,
                                        "city": null,
                                        "district": null,
                                        "geo_position": null,
                                        "house_number": null,
                                        "street": null
                                    },
                                    "lower_body": null,
                                    "mask": null,
                                    "match_result": null,
                                    "meta": null,
                                    "source": null,
                                    "stream_id": null,
                                    "tags": null,
                                    "top_match": null,
                                    "track_id": null,
                                    "upper_body": null,
                                    "user_data": ""
                                },
                                "similarity": 1.0
                            }
                        ]
                    }
                ],
                "reference": {
                    "id": ...,
                    "type": "event"
                }
            }
        ]
        """
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/bodies", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

    def matchRaw(
        self,
        candidates: List[BinaryReference],
        references: List[BinaryReference],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher raw API allows to match each candidate with each reference descriptors.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchingRaw

        Args:
            candidates: list of BinaryReference objects for candidates
            references: list of BinaryReference objects for references
            accountIdFilter: account id to match raw
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching for each reference.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.common.http_objs import RawDescriptor
        >>> from luna3.python_matcher.match_objects import RawDescriptorReference
        >>> from testData.resources import DESCRIPTOR
        >>>
        >>> response = client.matchRaw(
        ...     references=[RawDescriptorReference("raw_descriptor_references", DESCRIPTOR)],
        ...     candidates=[RawDescriptorReference("raw_descriptor_candidates", DESCRIPTOR)],
        ... )
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))
        {
            "matches": [
                {
                    "matches": [
                        {
                            "candidate_id": "raw_descriptor_candidates",
                            "similarity": 1.0
                        }
                    ],
                    "reference_id": "raw_descriptor_references"
                }
            ]
        }

        """
        jsonData = {
            "candidates": [reference.asDict(encode=False) for reference in candidates],
            "references": [reference.asDict(encode=False) for reference in references],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/raw", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

==> luna3-2.14.76-py3-none-any.whl: luna3/lunavl/mixins/matcher.py <==
"""
Module realize matcher mixin
"""

from typing import Awaitable, List, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload
from luna3.lunavl.mixins.base import MixinStub
from luna3.python_matcher.match_objects import BinaryReference, Candidates, PlatCoef, Reference


class MatcherMixin(MixinStub):
    """Matcher Mixin to work with matcher."""

    def matchFaces(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchFaces

        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, attributes, faces)
            accountIdFilter: account id to match faces
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching (failed and succeeded matches)

        >>> import json
        >>> from testData.resources import FACE_WARP
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.python_matcher.match_objects import FaceFilters
        >>> from luna3.python_matcher.match_objects import Reference
        >>> from luna3.python_matcher.match_objects import Candidates
        >>>
        >>> sampleId = client.saveFaceSample(FACE_WARP, raiseError=True).json["sample_id"]
        >>> attributeId = client.extractAttrFromSample(sampleIds=[sampleId], raiseError=True).json[0]["attribute_id"]
        >>> faceId = client.createFace(attributeId=attributeId, raiseError=True).json["face_id"]
        >>> response = client.matchFaces(
        ...     candidates=[Candidates(FaceFilters(faceIds=[faceId]), limit=3, threshold=0.5)],
        ...     references=[Reference("face", faceId)]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        [
            {
                "matches": [
                    {
                        "filters": {
                            "face_ids": [...],
                            "origin": "faces"
                        },
                        "result": [
                            {
                                "face": {
                                    "account_id": ...,
                                    "avatar": "",
                                    "create_time": ...,
                                    "event_id": null,
                                    "external_id": "",
                                    "face_id": ...,
                                    "lists": [],
                                    "user_data": ""
                                },
                                "similarity": 1.0
                            }
                        ]
                    }
                ],
                "reference": {
                    "id": ...,
                    "type": "face"
                }
            }
        ]

        """
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/faces", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

    match = matchFaces  # COMPAT

    def matchBodies(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher API allows to submit tasks to a service that searches for human bodies similar to a
        given reference(s) by matching them. A body attribute descriptor should be extracted for
        reference(s) and candidate(s).

        The sources for references are events or binary descriptor. The sources for candidates is events.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchBodies

        Args:
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: event, sdk_descriptor, raw_descriptor)
            accountIdFilter: account id to match bodies
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching (failed and succeeded matches).

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from testData.resources import BODY_WARP
        >>> from luna3.common.http_objs import Policies, DetectPolicy, ExtractPolicy
        >>> from luna3.python_matcher.match_objects import EventFilters
        >>> from luna3.python_matcher.match_objects import Reference
        >>> from luna3.python_matcher.match_objects import Candidates
        >>>
        >>> detectPolicy = DetectPolicy(detectBody=1, detectFace=0)
        >>> extractPolicy = ExtractPolicy(extractBodyDescriptor=1, extractFaceDescriptor=0)
        >>> policies = Policies(detectPolicy=detectPolicy, extractPolicy=extractPolicy)
        >>> handlerId = client.createHandler(
        ...     description="happy_people", policies=policies, raiseError=True
        ... ).json["handler_id"]
        >>> eventId = client.emitEvents(
        ...     handlerId=handlerId, inputData=BODY_WARP, raiseError=True,
        ... ).json['events'][0]['event_id']
        >>> response = client.matchBodies(
        ...     candidates=[Candidates(EventFilters(eventIds=[eventId]), limit=1, threshold=0.5)],
        ...     references=[Reference("event", eventId)]
        ... )
        >>> print(response.statusCode)
        200
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))  # doctest: +ELLIPSIS
        [
            {
                "matches": [
                    {
                        "filters": {
                            "event_ids": [...],
                            "origin": "events"
                        },
                        "result": [
                            {
                                "event": {
                                    "accessories": null,
                                    "account_id": ...,
                                    "age": null,
                                    "attach_result": [],
                                    "body_basic_attributes": null,
                                    "body_detections": [
                                        {
                                            "detect_time": ...,
                                            "detect_ts": null,
                                            "detection": {
                                                "rect": {
                                                    "height": ...,
                                                    "width": ...,
                                                    "x": ...,
                                                    "y": ...
                                                }
                                            },
                                            "image_origin": null,
                                            "sample_id": ...
                                        }
                                    ],
                                    "create_time": ...,
                                    "deepfake": null,
                                    "emotion": null,
                                    "end_time": ...,
                                    "ethnic_group": null,
                                    "event_id": ...,
                                    "external_id": "",
                                    "face_detections": null,
                                    "face_id": null,
                                    "gender": null,
                                    "handler_id": ...,
                                    "liveness": null,
                                    "location": {
                                        "area": null,
                                        "city": null,
                                        "district": null,
                                        "geo_position": null,
                                        "house_number": null,
                                        "street": null
                                    },
                                    "lower_body": null,
                                    "mask": null,
                                    "match_result": null,
                                    "meta": null,
                                    "source": null,
                                    "stream_id": null,
                                    "tags": null,
                                    "top_match": null,
                                    "track_id": null,
                                    "upper_body": null,
                                    "user_data": ""
                                },
                                "similarity": 1.0
                            }
                        ]
                    }
                ],
                "reference": {
                    "id": ...,
                    "type": "event"
                }
            }
        ]
        """
        jsonData = {
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/bodies", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

    def matchRaw(
        self,
        candidates: List[BinaryReference],
        references: List[BinaryReference],
        accountIdFilter: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Matcher raw API allows to match each candidate with each reference descriptors.

        See https://docs.visionlabs.ai/luna/latest/ReferenceManuals/APIReferenceManual.html#operation/matchingRaw

        Args:
            candidates: list of BinaryReference objects for candidates
            references: list of BinaryReference objects for references
            accountIdFilter: account id to match raw
            accept: expected content type

        Returns:
            LunaResponse: body contains json with results of matching for each reference.

        >>> import json
        >>> from luna3.lunavl.mixins.doctests.config import TEST_CLIENT as client
        >>> from luna3.common.http_objs import RawDescriptor
        >>> from luna3.python_matcher.match_objects import RawDescriptorReference
        >>> from testData.resources import DESCRIPTOR
        >>>
        >>> response = client.matchRaw(
        ...     references=[RawDescriptorReference("raw_descriptor_references", DESCRIPTOR)],
        ...     candidates=[RawDescriptorReference("raw_descriptor_candidates", DESCRIPTOR)],
        ... )
        >>> print(response.statusCode)
        201
        >>> print(json.dumps(response.json, sort_keys=True, indent=4))
        {
            "matches": [
                {
                    "matches": [
                        {
                            "candidate_id": "raw_descriptor_candidates",
                            "similarity": 1.0
                        }
                    ],
                    "reference_id": "raw_descriptor_references"
                }
            ]
        }

        """
        jsonData = {
            "candidates": [reference.asDict(encode=False) for reference in candidates],
            "references": [reference.asDict(encode=False) for reference in references],
        }
        queries = {"account_id": accountIdFilter}
        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/raw", "POST", body=payload, queryParams=queries, accept=accept, **kwargs
        )

    def matchGeneralEvents(
        self,
        candidates: List[Candidates],
        references: List[Union[Reference, BinaryReference]],
        eventType: str,
        descriptorType: str,
        platCoef: Optional[List[Union[dict, PlatCoef]]] = None,
        descriptorVersion: Optional[int] = None,
        accountId: Optional[str] = None,
        accept: str = "application/msgpack",
        **kwargs,
    ):
        """
        Matcher API allows to submit tasks to a service that searches for faces similar to a given reference(s)
        by matching them. An attribute descriptor should be extracted for reference(s) and candidate(s).
        A source for references are events, attributes and faces. A source for candidates are faces and lists.


        Args:
            eventType: base event_type for references and candidates
            descriptorType: base descriptor type for references and candidates
            descriptorVersion: descriptor version of matching
            platCoef: plat coefs, for normalizing matching results
            candidates: Candidates object with info about candidates
            references: list of Reference objects. Each object has entity id and it type
                        (possible type variants: events, attributes, faces, raw descriptors)
            accountId: account id
            accept: expected content type

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of matching (failed and succeeded matches)

        """
        queries = {"account_id": accountId}
        jsonData = {
            "event_type": eventType,
            "descriptor_type": descriptorType,
            "candidates": [candidate.asNoneEmptyDict() for candidate in candidates],
            "references": [
                reference.asDict() if isinstance(reference, Reference) else reference.asDict(encode=False)
                for reference in references
            ],
        }
        if platCoef:
            jsonData["plat_coef"] = platCoef if isinstance(platCoef, dict) else platCoef.asDict()
        if descriptorVersion:
            jsonData["descriptor_version"] = descriptorVersion

        payload = RequestPayload.buildMsgpack(body=jsonData)
        return self.makeRequest(
            f"{self.baseUri}/matcher/general_events", "POST", queryParams=queries, body=payload, accept=accept, **kwargs
        )
